import logging
import sys
import warnings

from sqlalchemy import exc as sa_exc, asc, desc

from schema import ArchiveDBSession, create_session, ExecutionBlock
from schema.model import Project, Author
from support.capo import get_my_capo_config
from support.logging import get_console_logger

from . import Telescope

_APPLICATION_NAME = 'project_fetcher'
_LOG = get_console_logger(_APPLICATION_NAME, logging.DEBUG)
_MISSING_PROFILE = """ERROR: unknown 'profile'; provide the -P argument 
or set the CAPO_PROFILE environment variable."""
_DESCRIPTION = """A tool for retrieving project metadata from the archive."""

class ArchiveProjectFetcher:

    def __init__(self, profile):
        self.capo_config = get_my_capo_config(profile=profile)
        try:
            self.archive_context = create_session('SDM',
                                                  profile=self.capo_config.profile)
            self.pst_context = ArchiveDBSession('PST', profile=self.capo_config.profile)
        except KeyError as k_ex:
            _LOG.error(f'An error occurred while creating a db context: {k_ex}')
            sys.exit(1)

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.archive_context.close()
        self.pst_context.session.close()

    def fetch_project(self, project_code: str):

        with warnings.catch_warnings():
            # Suppress SQLAlchemy warnings
            warnings.simplefilter("ignore", category=sa_exc.SAWarning)
            """
            Return the project specified by the input arguments, if it exists.
            :return: the first Project we found with the project code passed in
            """
            self.project = self.archive_context.query(Project) \
                .filter(Project.project_code == project_code) \
                .first()

            if self.project is None:
                raise AttributeError(f'project {project_code} not found')

            self.abstract = self.project.abstract
            # self.is_alma = self._is_alma()

            return self.project

    def build_project_info(self):
        ''' assemble the output '''
        output = []
        output.append(f'Title: {self.project.title}')
        output.append(f'Abstract: {self.project.abstract}')
        if self._is_alma():
            return

        investigator_list = self.project.authors
        if not investigator_list:
            raise ValueError(f'no authors found for {self.project.project_code}')

         # We want the PI's pst_person_id followed by the CoIs'
        # pst_person_ids in numeric order.
        # ALMA authors, however, do not have pst_person_ids
        project_pi = investigator_list[0]
        coi_pst_ids = [int(coi.pst_person_id) for coi in investigator_list[1:]]
        coi_pst_ids = sorted(coi_pst_ids)

        author_pst_ids = [int(project_pi.pst_person_id)]
        [author_pst_ids.append(id) for id in coi_pst_ids]
        authors_to_print = [str(id) for id in author_pst_ids]
        id_list = ' '.join(authors_to_print)
        output.append(f'Authors: {id_list}')

        return output

    def _get_investigators(self):
        """
         Get a list of investigators associated with this project,
         with PI(s) as first element(s) of the list

         :return: a list of investigators associated with the project code passed in,
         ordered with the PI(s) first
        """
        with warnings.catch_warnings():
            # Suppress SQLAlchemy warnings
            warnings.simplefilter("ignore", category=sa_exc.SAWarning)

            investigators_list = self.archive_context.query(Author) \
                .filter(Author.project == self.project) \
                .order_by(desc(Author.is_pi), asc(Author.pst_person_id)) \
                .all()
        return investigators_list

    def _is_alma(self):
        ''' is this an alma project? '''
        with warnings.catch_warnings():
            # Suppress SQLAlchemy warnings
            warnings.simplefilter("ignore", category=sa_exc.SAWarning)

            exec_block = self.archive_context.query(ExecutionBlock) \
                .filter(ExecutionBlock.project == self.project) \
                .filter(ExecutionBlock.telescope == Telescope.ALMA.value) \
                .first()
            return exec_block is not None
